/*
 * @Author: 抠脚本人
 * @QQ: 742374184
 * @Date: 2022-12-14 09:46:33
 * @LastEditTime: 2022-12-25 13:01:18
 * @Description: 界面管理工具模块
 * 灵感来之不易,积累创造奇迹
 */
import { Activity, setMainActivity, startActivity } from "ui";
import { EventEmitter } from "events";
import { Color } from "color";
//可以把常用的放在ui中，这里只用了一个aliveActivity
const ui = {};
//存放有效的activity
/**@type {Set<BaseActivity>} */
ui.aliveActivity = new Set();
//activity事件通知
class ActivityEvent extends EventEmitter {}
//基础界面类，用于记录有效的activity
class BaseActivity extends Activity {
	constructor() {
		super();
		this.valueMap = new Map();
		this.emitter = new ActivityEvent();
	}
	onBackPressed() {
		const simpleEvent = { consumed: false };
		this.emitter.emit("back_pressed", simpleEvent);
		if (!simpleEvent.consumed) {
			super.onBackPressed();
		}
	}
	/**
	 *
	 * @param {number} requestCode
	 * @param {number} resultCode
	 * @param {android.content.Intent} data
	 */
	onActivityResult(requestCode, resultCode, data) {
		this.emitter.emit("activity_result", requestCode, resultCode, data);
		super.onActivityResult(requestCode, resultCode, data);
	}
	/**
	 *
	 * @param {android.view.Menu} menu
	 * @returns
	 */
	onCreateOptionsMenu(menu) {
		this.emitter.emit("create_options_menu", menu);
		return menu.size() > 0;
	}
	/**
	 *
	 * @param {android.view.MotionEvent} motionEvent
	 */
	onGenericMotionEvent(motionEvent) {
		this.emitter.emit("generic_motion_event", motionEvent);
		super.onGenericMotionEvent(motionEvent);
	}
	/**
	 *
	 * @param {android.content.Intent} intent
	 */
	onNewIntent(intent) {
		this.emitter.emit("new_intent", intent);
		super.onNewIntent(intent);
	}
	/**
	 *
	 * @param {android.view.MenuItem} item
	 * @returns
	 */
	onOptionsItemSelected(item) {
		const simpleEvent = { consumed: false };
		this.emitter.emit("options_item_selected", item, simpleEvent);
		return simpleEvent.consumed || super.onOptionsItemSelected(item);
	}
	onPause() {
		this.emitter.emit("pause");
		super.onPause();
	}
	/**
	 *
	 * @param {number} requestCode
	 * @param {Array<string>} permissions
	 * @param {Array<number>} grantResults
	 */
	onRequestPermissionsResult(requestCode, permissions, grantResults) {
		super.onRequestPermissionsResult(requestCode, permissions, grantResults);
		this.emitter.emit("request_permission_result", requestCode, permissions, grantResults);
	}
	onRestart() {
		super.onRestart();
		this.emitter.emit("restart");
	}
	onResume() {
		super.onResume();
		this.emitter.emit("resume");
	}
	/**
	 * @param {android.os.Bundle | undefined} instance
	 */
	onCreate(instance) {
		ui.aliveActivity.add(this);
		super.onCreate(instance);
	}
	onDestroy() {
		ui.aliveActivity.delete(this);
		super.onDestroy();
	}
	/**
	 * 设置状态栏底色
	 * @param {string|number} color 颜色文本或颜色值
	 */
	setStatusBarColor(color) {
		let colorNumber = color;
		if (typeof color !== "number") {
			colorNumber = Color.parse(color)?.value ?? -1;
		}
		this.getWindow().setStatusBarColor(/** @type {number} */ (colorNumber));
	}
	/**
	 * 调整状态栏颜色
	 * @param {boolean} light 是否为亮色状态栏
	 */
	setLightStatusBar(light) {
		const insetsController = this.insetsController;
		insetsController.setAppearanceLightStatusBars(light);
	}
}
/**
 * 创建界面函数,可多次调用创建多个activity
 * @param {string} xml 要显示的界面布局
 * @param {{configs?:import("ui").StatusBarConfig,scope?:object}} options
 * @returns {Promise<BaseActivity>}
 */
ui.layout = function (xml, options = {}) {
	return new Promise((resolve, _reject) => {
		const CurrentActivity = class extends BaseActivity {
			get initialStatusBar() {
				return options.configs ?? { color: "#ffffff", light: true };
			}

			get layoutXml() {
				/**@type {Record<string, any>} */
				let scope = options.scope ?? globalThis;
				let code = `
				let {${Object.keys(scope)}} = scope
				return xml.replace(/{{(.+?)}}/g, (_match, key) => {
					return eval(key) 
				});`;
				let fun = new Function("xml", "scope", code);
				return fun(xml, scope);
			}
			/**
			 *
			 * @param {import("ui").JsView<android.view.ViewGroup>} view
			 */
			onContentViewSet(view) {
				//可以再此实现声明式组件
				// view.getChildCount();
				resolve(this);
			}
		};
		if (ui.aliveActivity.size == 0) {
			setMainActivity(CurrentActivity);
		} else {
			startActivity(CurrentActivity);
		}
	});
};
globalThis.ui = ui;
export default ui;
